/* libutf8++/src/lib/encoder.cpp
 *
 *  (c)2006, Laurence Withers. Released under the GNU GPL. See file
 *  COPYING for more information / terms of license.
*/

namespace utf8 {



Encoder::Encoder(size_t hint)
{
    memset(&ctx, 0, sizeof(ctx));
    ctx.wr_size = (hint < 7) ? 7 : hint;
    ctx.wr = new char[ctx.wr_size];
    ctx.error_callback = _exceptionOnError;
    ctx.data = this;
}



Encoder::~Encoder()
{
    delete [] ctx.wr;
}



void Encoder::reset()
{
    char* wr = ctx.wr;
    size_t wr_size = ctx.wr_size;
    utf8_encode_error_callback cb = ctx.error_callback;
    memset(&ctx, 0, sizeof(ctx));
    ctx.wr = wr;
    ctx.wr_size = wr_size;
    ctx.error_callback = cb;
    ctx.data = this;
}



void Encoder::encode(const std::wstring& str)
{
    encode(str.data(), str.size());
}



void Encoder::encode(const wchar_t* str, ssize_t amt)
{
    ctx.rd = str;
    ctx.rd_remain = amt;
    while(ctx.rd_remain) {
        if(!utf8_encoder(&ctx)) throw BadUnicodeChar(&ctx);
        encoded.append(ctx.wr, ctx.written);

        if(ctx.rd_remain < 0 && !*(ctx.rd)) break;
        if(ctx.rd_remain) {
            ctx.wr_size *= 2;
            delete [] ctx.wr;
            ctx.wr = new char[ctx.wr_size];
        }
    }
}



void Encoder::skipOnError()
{
    ctx.error_callback = _skipOnError;
}



void Encoder::replaceOnError(wchar_t ch)
{
    replaceChar = ch;
    ctx.error_callback = _replaceOnError;
}



void Encoder::exceptionOnError()
{
    ctx.error_callback = _exceptionOnError;
}



enum utf8_encode_error_action Encoder::_skipOnError
    (const struct utf8_encode_state *ctx, wchar_t *newch)
{
    (void)ctx;
    (void)newch;
    return utf8_encode_error_action_skip;
}



enum utf8_encode_error_action Encoder::_replaceOnError
    (const struct utf8_encode_state *ctx, wchar_t *newch)
{
    Encoder* self = (utf8::Encoder*)(ctx->data);
    *newch = self->replaceChar;
    return utf8_encode_error_action_replace;
}



enum utf8_encode_error_action Encoder::_exceptionOnError
    (const struct utf8_encode_state *ctx, wchar_t *newch)
{
    (void)newch;
    throw BadUnicodeChar(ctx);
}



}

/* options for text editors
kate: replace-trailing-space-save true; space-indent true; tab-width 4;
*/
