/* libutf8++/src/lib/exception.cpp
 *
 *  (c)2006, Laurence Withers. Released under the GNU GPL. See file
 *  COPYING for more information / terms of license.
*/

namespace utf8 {



Error::Error(const std::string& reason)
    : reason(reason)
{
}



const char* Error::what()
{
    return reason.c_str();
}



BadUnicodeChar::BadUnicodeChar(const struct utf8_encode_state* ctx)
    : Error(format(ctx)), badChar(*ctx->rd), line(ctx->line), col(ctx->col), char_offset(ctx->char_offset)
{
}



std::string BadUnicodeChar::format(const struct utf8_encode_state* ctx)
{
    std::ostringstream str;

    str << "Invalid Unicode code point encountered."
           "\n  Position       : line "
        << ctx->line + 1
        << ", column "
        << ctx->col + 1
        << "\n  Stream offset  : "
        << ctx->char_offset
        << " characters\n  Character value: 0x"
        << std::hex
        << *(ctx->rd);

    return str.str();
}



BadUTF8Sequence::BadUTF8Sequence(const std::string& description,
                                 const struct utf8_decode_state* ctx)
    : Error(format(description, ctx)), description(description), line(ctx->line + 1),
    col(ctx->col + 1), char_offset(ctx->char_offset), byte_offset(ctx->byte_offset)
{
}



std::string BadUTF8Sequence::format(const std::string& description,
                                    const struct utf8_decode_state* ctx)
{
    std::ostringstream str;

    str << "Bad byte sequence in UTF-8 data.\n"
        "  Reason  : " << description
        << "\n  Position: line " << ctx->line + 1
        << ", column " << ctx->col + 1
        << "\n  Offset  : " << ctx->char_offset << " chars, " << ctx->byte_offset << " bytes";

    return str.str();
}



}

/* options for text editors
kate: replace-trailing-space-save true; space-indent true; tab-width 4;
*/
