/* libutf8++/src/lib/string.cpp
 *
 *  (c)2006, Laurence Withers. Released under the GNU GPL. See file
 *  COPYING for more information / terms of license.
*/

namespace utf8 {



static enum utf8_decode_error_action decode_replace_callback(const struct utf8_decode_state* ctx,
    enum utf8_decode_error error, wchar_t* newch)
{
    (void)error;
    *newch = *(wchar_t*)(ctx->data);
    return utf8_decode_error_action_replace;
}



static enum utf8_decode_error_action decode_error_callback(const struct utf8_decode_state* ctx,
    enum utf8_decode_error error, wchar_t* newch)
{
    (void)newch;
    const char* desc = "unknown";

    switch(error) {
    case utf8_decode_error_lone_cchar:
        desc = "An invalid continuation byte was encountered while expecting a character.";
        break;

    case utf8_decode_error_not_cchar:
        desc = "A multi-byte sequence contained an invalid byte.";
        break;

    case utf8_decode_error_not_schar:
        desc = "An invalid byte was encountered while expecting a character.";
        break;

    case utf8_decode_error_overlong:
        desc = "An overlong encoding of a character was encountered.";
        break;

    case utf8_decode_error_illegal_cp:
        desc = "An illegal code point (a UTF-16 surrogate perhaps?) was encountered.";
        break;
    }

    throw BadUTF8Sequence(desc, ctx);
}



std::wstring decode(const std::string& utf8, bool force, wchar_t replace)
{
    wchar_t buffer[128];
    struct utf8_decode_state ctx;
    memset(&ctx, 0, sizeof(ctx));

    ctx.rd = utf8.data();
    ctx.rd_remain = utf8.size();
    ctx.wr = buffer;
    ctx.wr_size = 128;
    if(force) {
        ctx.error_callback = decode_replace_callback;
        ctx.data = &replace;
    } else {
        ctx.error_callback = decode_error_callback;
    }

    std::wstring ret;

    while(ctx.rd_remain) {
        utf8_decoder(&ctx);
        ret.append(buffer, ctx.written);
    }

    return ret;
}



static enum utf8_encode_error_action encode_replace_callback(const struct utf8_encode_state* ctx,
    wchar_t* newch)
{
    *newch = *(wchar_t*)(ctx->data);
    return utf8_encode_error_action_replace;
}



std::string encode(const std::wstring& ustr, bool force, wchar_t replace)
{
    char buffer[512];
    struct utf8_encode_state ctx;
    memset(&ctx, 0, sizeof(ctx));

    ctx.rd = ustr.data();
    ctx.rd_remain = ustr.size();
    ctx.wr = buffer;
    ctx.wr_size = 512;
    if(force) {
        ctx.error_callback = encode_replace_callback;
        ctx.data = &replace;
    }

    std::string ret;

    while(ctx.rd_remain) {
        if(!utf8_encoder(&ctx)) {
            throw BadUnicodeChar(&ctx);
        }

        ret.append(buffer, ctx.written);
    }

    return ret;
}



}

/* options for text editors
kate: replace-trailing-space-save true; space-indent true; tab-width 4;
*/
