/* libutf8++/src/tests/strings.cpp
 *
 *  (c)2006, Laurence Withers. Released under the GNU GPL. See file
 *  COPYING for more information / terms of license.
*/

#include "utf8"
#include <iostream>
#include <iomanip>
#include <fcntl.h>
#include <unistd.h>



void make_random(wchar_t* buf, int ch)
{
    int fd = open("/dev/urandom", O_RDONLY);
    if(fd < 0) {
        perror("open(\"/dev/urandom\")");
        throw 1;
    }
    ch *= sizeof(wchar_t);
    if(read(fd, (char*)buf, ch) != ch) {
        perror("read(\"/dev/urandom\")");
        throw 1;
    }
    close(fd);

    ch /= sizeof(wchar_t);
    while(ch--) {
        buf[ch] &= 0x7FFFFFFF;
    }
}



int main(int argc, char* argv[])
{
    if(argc == 2 && !strcmp(argv[1], "--print-summary")) {
        std::cout << "Performs some tests on the string encode/decode routines.\n";
        return 0;
    }

    int ret = 0;
    try {
        wchar_t wch[1024];
        make_random(wch, 1024);

        std::wstring ustr1, ustr2;
        std::string utf8;
        ustr1.assign(wch, 1024);
        utf8 = utf8::encode(ustr1);
        ustr2 = utf8::decode(utf8);

        if(ustr1 != ustr2) {
            std::cerr << "Decoded string does not match original.\n";
            for(size_t i = 0, end = std::min(ustr1.size(), ustr2.size()); i != end; ++i) {
                if(ustr1[i] != ustr2[i]) {
                    std::cerr << std::dec << std::setfill(' ') << std::setw(4) << i
                            << std::setfill('0') << std::hex << ": 0x"
                            << std::setw(8) << ustr1[i] << " != "
                            << std::setw(8) << ustr2[i] << "\n";
                }
            }
            std::cerr << "Original size " << std::dec << ustr1.size()
                    << ", decoded size " << ustr2.size() << std::endl;
            return 1;
        }

        std::cout << "Success.\n";
    }
    catch(utf8::Error& e) {
        std::cerr << e.reason << std::endl;
        ret = 1;
    }

    return ret;
}

/* options for text editors
kate: replace-trailing-space-save true; space-indent true; tab-width 4;
*/
